//------------------------------------------------------------------------------
//
//  npgldraw.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------

#include "npgldraw.h"

#include <stdlib.h>

#include "../npglut.h"
#include "../../data/npplot.h"
#include "../../npctrl.h"


#define kTorusScreenSizeCount 4
#define kTorusThicknessCount 4

//MB//
extern int selectedID;
int pickPass;				//zz move it to global data ptr, debug
int selectedTree;
int selectTreeID;


void DrawDefault (NPplotPtr plot);
void DrawCamera (NPplotPtr plot);
void DrawSurface (NPplotPtr plot);
void DrawPoints (NPplotPtr plot);
void DrawPin (int selected, NPplotPtr plot, void* dataRef);
void DrawPinChild (NPplotPtr plot, void* dataRef);
void DrawVideo (NPplotPtr plot);
void DrawGrid (NPplotPtr plot);


struct NPtorus {
	NPfloatXYZPtr	vArray;
	GLint			vArraySize;
	GLint*			quadArray;
	GLint			quadArraySize;
	int				slices;
	int				segments;
	GLfloat			thickness;
};
typedef struct NPtorus NPtorus;
typedef struct NPtorus * NPtorusPtr;

struct NPcircle {
	NPfloatXYPtr	vArray;
	int				segments;
	GLfloat			radius;
};
typedef struct NPcircle NPcircle;
typedef struct NPcircle * NPcirclePtr;


void InitTorus (void* dataRef);


static GLint pin_dl;
static GLint pin_wire_dl;
static GLint pin_torus_dl;
static GLint pin_torus_wire_dl;


GLuint CreatePinDL();
GLuint CreatePinWireDL();
void DrawPinDL();
void DrawPinWireDL();

GLuint CreateTorusDL();
GLuint CreateTorusWireDL();
void DrawTorusDL();
void DrawTorusWireDL(); 

void npDrawTorus (GLfloat radius, GLfloat thickness);

void CreateTorus (NPtorusPtr torus);
void CreateCircle (NPcirclePtr circle);
void DeleteCircle (NPcirclePtr circle);



//------------------------------------------------------------------------------
void npInitGLDraw(void* dataRef)
{
//	InitTorus (dataRef);
	pickPass = false;			//zz global init

	pin_dl = CreatePinDL();
	pin_wire_dl = CreatePinWireDL();
	pin_torus_dl = CreateTorusDL();
	pin_torus_wire_dl = CreateTorusWireDL();
}


// creates a set of toroids and loads them into GPU memory
//------------------------------------------------------------------------------
void InitTorus (void* dataRef)
{
	int i = 0, j = 0;
	NPtorusPtr torus = NULL;

	torus = (NPtorusPtr) malloc (sizeof(NPtorus));
	if (torus == NULL)
	{
		printf ("\n 4220 error malloc failed cannot write file \n");
		return;
	}

	// 5x5, 7x5, 12x6, 16x6, 24x8, 32x10, 48x12, 64x16, 96x24, 128x32, 196x48
	for ( i = 0; i < kTorusScreenSizeCount; i++)
		for ( j = 0; j < kTorusThicknessCount; j++)
		{
			switch (i)
			{
				case 0 : torus->slices = 5; torus->segments = 5; break;
				case 1 : torus->slices = 5; torus->segments = 7; break;
				case 2 : torus->slices = 6; torus->segments = 12; break;
				case 3 : torus->slices = 6; torus->segments = 16; break;
				case 4 : torus->slices = 8; torus->segments = 24; break;
				case 5 : torus->slices = 10; torus->segments = 32; break;
				case 6 : torus->slices = 12; torus->segments = 48; break;
				case 7 : torus->slices = 16; torus->segments = 64; break;
				case 8 : torus->slices = 24; torus->segments = 96; break;
				case 9 : torus->slices = 32; torus->segments = 128; break;
				case 10 : torus->slices = 48; torus->segments = 196; break;
				case 11 : torus->slices = 64; torus->segments = 256; break;
			}
			
			torus->thickness =	(GLfloat) j * 100.0f / 
								(GLfloat) (kTorusThicknessCount - 1);

			CreateTorus (torus);
			// load torus to GPU and deallocate memory
			// we could save time by allocating enough memory for all torus's at once
			// or enough for largest torus then de-allocating all at once
		}
}


// creates a single torus of specified slices, stacks and thickness
//------------------------------------------------------------------------------
void CreateTorus (NPtorusPtr torus)
{
/*	int i=0, j=0;

	NPcirclePtr ring = (NPcirclePtr) malloc (sizeof(NPcircle));
	NPcirclePtr hoops = (NPcirclePtr) malloc (torus->slices * sizeof(NPcircle));

	torus->vArraySize = torus->slices * torus->segments;
	torus->vArray = (NPfloatXYZPtr) malloc (torus->vArraySize * sizeof(NPfloatXYZ));

	torus->quadArraySize = torus->vArraySize * 4;
	torus->quadArray = (GLint*) malloc (torus->quadArraySize * sizeof(GLint));

	// create our tube ring
	ring->radius = torus->thickness;		// 1.0 is no hole in the donut
	ring->segments = torus->slices;
	CreateCircle (ring);

	// create our hoops, the length segments of the tube
	for (i=0; i < torus->slices; i++)
	{
		hoops[i].radius = 1.0f + ring->vArray[i].x;
		hoops[i].segments = torus->segments;
		CreateCircle (&hoops[i]);
	}

	// build the vertex array for the torus
	for (i=0; i < torus->segments; i++)
		for (j=0; j < torus->slices; j++)
		{
			torus->vArray[i*torus->slices + j].x = hoops[j].vArray[i].x;		//the hoop provides x and y
			torus->vArray[i*torus->slices + j].y = hoops[j].vArray[i].y;
			torus->vArray[i*torus->slices + j].z = ring->vArray[j].y;		//the ring provides the z
		}

	// build the the torus polygon array
	for (i=0; i < torus->segments; i++)
		for (j=0; j < torus->slices; j++)
		{	
			if ( i == (torus->segments - 1) )		// the last torus segment
			{
				if ( j == (torus->slices - 1) )		// the last polygon
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices);
					torus->quadArray[i][j][2] = 0;
					torus->quadArray[i][j][3] = j;
				}
				else
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices) + (j + 1);
					torus->quadArray[i][j][2] = (j + 1);
					torus->quadArray[i][j][3] = j;
				}
			}
			else
			{
				if ( j == (torus->slices - 1) )		// the last polygon in a segment
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices);
					torus->quadArray[i][j][2] = ((i + 1) * torus->slices);
					torus->quadArray[i][j][3] = ((i + 1) * torus->slices) + j;
				}
				else
				{
					torus->quadArray[i][j][0] = (i * torus->slices) + j;
					torus->quadArray[i][j][1] = (i * torus->slices) + (j + 1);
					torus->quadArray[i][j][2] = ((i + 1) * torus->slices) + (j + 1);
					torus->quadArray[i][j][3] = ((i + 1) * torus->slices) + j;
				}
			}

	DeleteCircle (ring);
	for (i=0; i < torus->slices; i++)
		DeleteCircle (&hoops[i]);
*/
}


// creates a circle of specified number of points and radius
// centered on the origin (0,0) starting at 12 o'clock going CW
// really should be part of GL, glCircle!, lol
//------------------------------------------------------------------------------
void CreateCircle (NPcirclePtr circle)
{
/*	int i = 0;
	GLfloat angle = 0.0, deltaAngle = 0.0f;

	circle->vArray = (NPfloatXY) malloc (cirlce->segments * sizeof(NPfloatXY));


	if (circle->segments)
		deltaAngle = (2.0 * kPI) / circle->segments;
	
	// go around the circle, 
	for (angle = 0.0f; angle <= 1.99999f * kPI; angle += deltaAngle)
	{
//		circle->vArray[i]->x = circle->radius * (GLfloat)sin(angle);
//		circle->vArray[i].y = circle->radius * (GLfloat)cos(angle);
		i++;
	}
*/
}


//------------------------------------------------------------------------------
void DeleteCircle (NPcirclePtr circle)
{
	free (circle->vArray);
	free (circle);
}


// draws a torus of specified size and thickness, stacks and slices determined
// by size in pixels on screen calculated by screen size and z distance
// radius from center of the donut to the center of the tube, thickness 0-100%
//------------------------------------------------------------------------------
void npDrawTorus (GLfloat radius, GLfloat thickness)
{
	int torusID = 0;

	// torusID = screenSize + (thickness / numberOfSizes);

	glPushMatrix();
		glScalef(radius, radius, radius);
		glCallList(torusID);
	glPopMatrix();
}


// Draw Pin, ice-cream cone shape
//------------------------------------------------------------------------------
void DrawPinDL() 
{
	glTranslatef (0.0f, 0.0f, 5.0f);
	glRotatef (180.0f, 0.0f, 1.0f, 0.0f);
	npGLSolidCone (0.5f, 5.0f, 10, 1);

	glRotatef (-180.0f, 0.0f, 1.0f, 0.0f);	//undo rotation n save on pushMatrix
	npGLSolidSphere (0.5f, 10, 10);
}

// Draw Pin, ice-cream cone shape
//------------------------------------------------------------------------------
void DrawPinWireDL() 
{
	glTranslatef (0.0f, 0.0f, 5.0f);
	glRotatef (180.0f, 0.0f, 1.0f, 0.0f);
	npGLWireCone (0.5f, 5.0f, 10, 1);

	glRotatef (-180.0f, 0.0f, 1.0f, 0.0f);	//undo rotation n save on pushMatrix
	npGLWireSphere (0.5f, 10, 10);
}


//------------------------------------------------------------------------------
void DrawTorusDL() 
{
	npGLSolidTorus (0.015f, 0.15f, 7, 16);
}


//------------------------------------------------------------------------------
void DrawTorusWireDL() 
{
	npGLWireTorus (0.015f, 0.15f, 7, 16);
}


//------------------------------------------------------------------------------
GLuint CreatePinDL()
{
	GLuint displayList;

	// Create the id for the list
	displayList = glGenLists(1);

	// start list
	glNewList (displayList, GL_COMPILE);

	// call the function that contains the rendering commands
	DrawPinDL();

	// endList
	glEndList();

	return (displayList);
}


//------------------------------------------------------------------------------
GLuint CreatePinWireDL()
{
	GLuint displayList;

	// Create the id for the list
	displayList = glGenLists(2);

	// start list
	glNewList (displayList, GL_COMPILE);

	// call the function that contains the rendering commands
	DrawPinWireDL();

	// endList
	glEndList();

	return (displayList);
}


//------------------------------------------------------------------------------
GLuint CreateTorusDL()
{
	GLuint displayList;

	// Create the id for the list
	displayList = glGenLists(3);

	// start list
	glNewList (displayList, GL_COMPILE);

	// call the function that contains the rendering commands
	DrawTorusDL();

	// endList
	glEndList();

	return (displayList);
}


//------------------------------------------------------------------------------
GLuint CreateTorusWireDL()
{
	GLuint displayList;

	// Create the id for the list
	displayList = glGenLists(4);

	// start list
	glNewList (displayList, GL_COMPILE);

	// call the function that contains the rendering commands
	DrawTorusWireDL();

	// endList
	glEndList();

	return (displayList);
}

						
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
//------------------------------------------------------------------------------
void npDrawPlots (void* dataRef)
{
	pData data = (pData) dataRef;
	NPplotPtr plot = NULL;

	selectTreeID = 0;
	selectedTree = 0;

	for (selectTreeID=0; selectTreeID < data->map.plotCount; selectTreeID++)
	{
		plot = data->map.plot[selectTreeID];

		if (data->map.plotIndex == selectTreeID)
			selectedTree = true;
		else
			selectedTree = false;

		if (!plot->hide)			// don't draw hidden plots
			switch (plot->type)
			{
				case kPlotDefault : DrawDefault (plot); break;
				case kPlotCamera : DrawCamera (plot); break;
				case kPlotSurface : DrawSurface (plot); break;
				case kPlotPoints : DrawPoints (plot); break;
				case kPlotPin : DrawPin (selectedTree, plot, data); break;
				case kPlotVideo : DrawVideo (plot); break;
				case kPlotGrid : DrawGrid (plot); break;
			}
	}
}


//------------------------------------------------------------------------------
void DrawDefault (NPplotPtr plot)
{

}


// draw an icon at the location of the camera in the scene
//------------------------------------------------------------------------------
void DrawCamera (NPplotPtr plot)
{
	// draw icon of camera in the scene, but not the current viewport camera
}


//------------------------------------------------------------------------------
void DrawSurface (NPplotPtr plot)
{
	// draw ground
	glPushMatrix();

	// position the plot
	glTranslatef (plot->translate.x, plot->translate.y, plot->translate.z);
	glRotatef (plot->rotate.angle, plot->rotate.x, plot->rotate.y, plot->rotate.z);
	glScalef (plot->scale.x, plot->scale.y, plot->scale.z);

	glColor4ub (plot->color.r, plot->color.g, plot->color.b, plot->color.a);

	glBegin(GL_QUADS);
		glVertex3f (-100.0f, -100.0f, 0.0f);
		glVertex3f (-100.0f,  100.0f, 0.0f);
		glVertex3f ( 100.0f,  100.0f, 0.0f);
		glVertex3f ( 100.0f, -100.0f, 0.0f);
	glEnd();

	glPopMatrix();
}


//------------------------------------------------------------------------------
void DrawPoints (NPplotPtr plot)
{

}


//MB//
void IDtoRGB( int id, int* r, int* g, int* b )
{
	int remain = id;
	
	*r = remain / (256*256);	remain = remain - *r * (256*256);
	*g = remain / 256;			remain = remain - *g * 256;
	*b = remain;
//	WATCH_INT( id );
//	WATCH_INT( *r );
//	WATCH_INT( *g );
//	WATCH_INT( *b );
}

int RGBtoID( int r, int g, int b )
{
	int id;
	id = (256*256)*r + 256*g + b;
//	WATCH_INT( id );
	return id;
}


void PickPass( int picking )
{
	pickPass = picking;
}
//MB-End//

//------------------------------------------------------------------------------
void DrawPin (int currentPlot, NPplotPtr plot, void* dataRef)
{
	int i = 0;
	
	int branchLevel = 0;

	pData data = (pData) dataRef;

	NPplotPtr plotChild = data->map.activePlot;

	//MB//	
	int plotID = plot->id;
	int idRed, idGrn, idBlu;
	int remain = 0;
	idRed = idGrn = idBlu = 0;

	//SelectById. Select from selectedID
	if (pickPass)
	{
		IDtoRGB( plotID, &idRed, &idGrn, &idBlu );	//Convert plotID to RGB triplet

		if (0)//data->io.key.modShift == true)				// zz
			plot->selected = false;

		if( selectedID == plot->id )
		{
			if (0)//data->io.key.modShift == true)
			{
			  plot->selected = 1 - plot->selected;
			}
			else
			{
				//clear selection
				npCtrlCommand (kGraphSelectAllToggle, data);	//zz debug not efficient
				plot->selected = true;	// toggle the selection
				data->map.selectedPlot = plot;
				data->map.activePlot = plot;
				selectedTree = true;
				currentPlot = true;
				data->map.plotIndex = selectTreeID;				//zz end
			}
		}
	}
	//MB-End//
	

	branchLevel = plotChild->branchLevel;

	glPushMatrix();

	// position the plot
	glTranslatef (plot->translate.x, plot->translate.y, plot->translate.z);
	glRotatef (plot->rotate.angle, plot->rotate.x, plot->rotate.y, plot->rotate.z);
	glScalef (plot->scale.x, plot->scale.y, plot->scale.z);

		//MB//
	//Main Color for Rendering Pin
	if( pickPass )
	{
		glDisable(GL_LIGHTING);
		glColor4ub( idRed, idGrn, idBlu, 255 );	//ID of Plot is encoded on the Pixel! In RGB Triplet   (Range: 0-  16.7 Million)

	//	pickPass = false;	//zz
	}
	else
	{
		glEnable(GL_LIGHTING);
		glColor4ub (plot->color.r, plot->color.g, plot->color.b, plot->color.a);
	}
	//-

//	glColor4ub (plot->color.r, plot->color.g, plot->color.b, plot->color.a);  //debug, zz
	
	if (plot->surfaceType == kSurfacePin)
		if (plot->primitive == kNPprimitivePin)
			glCallList (pin_dl);												//zz	debug
		else
			npGLPrimitive (plot->primitive);

	if (plot->selected || currentPlot)
	{
		glPushMatrix();

		if (currentPlot)
		{
			// slide down a little for scaling up to surround the pin
			glTranslatef (0.0f, 0.0f, -5.10f);
			glScalef (1.15f, 1.15f, 1.15f);

			if (plot->surfaceType == kSurfacePin)
			{
				if ( plot->selected )
				{
					glColor4f ( 0.8f, 0.8f, 0.0f, 1.0f);	// draw selected yellow
					glCallList (pin_wire_dl);
				}
				else if (branchLevel == 0)
				{
					glColor4f ( 1.0f, 0.0f, 0.0f, 1.0f);	// draw selection red
					glCallList (pin_wire_dl);
				}
				else
					glTranslatef (0.0f, 0.0f, 5.0f);		// move up to draw other

				if (branchLevel > 0)
				{
					if (plotChild->selected)
						glColor4f ( 0.8f, 0.8f, 0.0f, 1.0f);	// draw selected yellow
					else
						glColor4f ( 1.0f, 0.0f, 0.0f, 1.0f);	// draw selection red
				}

				glScalef (1.0f, 1.0f, 0.66f);
				glCallList (pin_wire_dl);				// 2nd one will be above
				
			}
		}
		else if (plot->selected)
		{
			glTranslatef (0.0f, 0.0f, -5.10f);
			glScalef (1.05f, 1.05f, 1.05f);

			glColor4f ( 0.8f, 0.8f, 0.0f, 1.0f);		// draw selected as yellow

			if (plot->surfaceType == kSurfacePin)
				glCallList (pin_wire_dl);
		}

		glPopMatrix();
	}

	//should call DrawPlot, that calls draw pin, move this to drawplots...   debug, zz
	for (i=0; i < plot->childCount; i++)
		DrawPinChild (plot->child[i], dataRef);


	glPopMatrix();
}

// first draw child here, then break it up, draw plots should iterate down, debug, zz
// the tree branches and take care of selected glows, etc...
// this routine should be generic regardless of branch level
//------------------------------------------------------------------------------
void DrawPinChild (NPplotPtr plot, void* dataRef)
{
	int i = 0;

	pData data = (pData) dataRef;

	NPpinPtr plotData = (NPpinPtr) plot->data;


	//MB//
//	pickPass = false;

	int plotID = plot->id;
	int idRed, idGrn, idBlu;
	int remain = 0;
	idRed = idGrn = idBlu = 0;
	
	//Convert plotID to RGB triplet
	IDtoRGB( plotID, &idRed, &idGrn, &idBlu );
//	plotID = RGBtoID( idRed, idGrn, idBlu );		//Check conversion correctness

	//SelectById. Select from selectedID
	if (pickPass)
	{
		if (0)//data->io.key.modShift == true)				// zz
			plot->selected = false;

		if( selectedID == plot->id )
		{
			if (0)//data->io.key.modShift == true)
				plot->selected = 1 - plot->selected;
			else
			{
				npCtrlCommand (kGraphSelectAllToggle, data);	//zz debug not efficient
				plot->selected = true;	// toggle the selection
				data->map.selectedPlot = plot;
				data->map.activePlot = plot;
				selectedTree = true;
	//			currentPlot = true;
				data->map.plotIndex = selectTreeID;				//zz end
			}
		}
	}
	//MB-End//


	if (plot->surfaceType == kSurfaceTorus)
	{
		if (plot->branchLevel > 1)
		{
			glRotatef (90.0f, 3.0f, 0.0f, 0.0f);
			glRotatef (plot->rotate.angle, 0.0f, 1.0f, 0.0f);
		}

		if (plot->branchLevel > 1 )
		{
			glTranslatef (1.5f, 0.0f, 0.0f);
			glScalef ( 0.5f * plot->scale.x, 0.5f * plot->scale.y, 0.5f * plot->scale.z);
		}
		else
			glScalef ( plot->scale.x, plot->scale.y, plot->scale.z);	//root child

				//MB//
	//Main Color for Rendering Pin
	if( pickPass )
	{
		glDisable(GL_LIGHTING);
		glColor4ub( idRed, idGrn, idBlu, 255 );	//ID of Plot is encoded on the Pixel! In RGB Triplet   (Range: 0-  16.7 Million)
	
//		pickPass = false;	//zz
	}
	else
	{
		glEnable(GL_LIGHTING);
		glColor4ub (plot->color.r, plot->color.g, plot->color.b, plot->color.a);
	}
//		glColor4ub (plot->color.r, plot->color.g, plot->color.b, plot->color.a); //debug, zz
//-

//		glPushMatrix();
	//		glScalef (10.0f, 10.0f, 10.0f);		//workaround for shading bug...
//			glCallList (pin_torus_dl);
			npGLPrimitive (plot->primitive);	//zz
//		glPopMatrix();
	

		for (i=0; i < plot->childCount; i++)
		{
			glPushMatrix();
				DrawPinChild (plot->child[i], data);
			glPopMatrix();	
		}

		//draw a wireframe slightly scaled up on top...
		if (plot->selected) // lineWidth > 0.0f)			//add feature, debug zz
		{	
			glPushMatrix();
				glLineWidth (plot->lineWidth);
				glColor4f (0.8f, 0.8f, 0.0f, 1.0f);			//draw yellow
				glScalef (10.3f, 10.3f, 10.3f);
				glCallList (pin_torus_wire_dl);
			glPopMatrix();
		}

		//draw a wireframe slightly scaled up on top...
		if (plot == data->map.selectedPlot) // lineWidth > 0.0f)			//add feature, debug zz
		{	
			glPushMatrix();
				glColor4f (1.0f, 0.0f, 0.0f, 1.0f);			//draw red
				glScalef (10.4f, 10.4f, 15.0f);
				glCallList (pin_torus_wire_dl);
			glPopMatrix();
		}

		//draw points slightly scaled up on top...
		if (plot->pointSize > 0.0f)
		{
			//draw points slightly scaled up on top...		//add feature, debug zz
		}
	}
}


//------------------------------------------------------------------------------
void DrawVideo (NPplotPtr plot)
{

}


//------------------------------------------------------------------------------
void DrawGrid (NPplotPtr plot)
{
	int			i = 0;
	int			j = 0;
	int			k = 0;
	GLfloat		x = 0.0f;
	GLfloat		y = 0.0f;
	GLfloat		z = 0.0f;
	NPfloatXYZ	length;

	NPgridPtr grid = (NPgridPtr) plot->data;

	length.x = grid->length.x * grid->segments.x;
	length.y = grid->length.y * grid->segments.y;
	length.z = grid->length.z * grid->segments.z;

	glPushMatrix();

	// position the plot
	glTranslatef (plot->translate.x, plot->translate.y, plot->translate.z);
	glRotatef (plot->rotate.angle, plot->rotate.x, plot->rotate.y, plot->rotate.z);
	glScalef (plot->scale.x, plot->scale.y, plot->scale.z);

	glTranslatef (length.x / -2.0f, length.y / -2.0f, 0.0f);

	glColor4ub (plot->color.r, plot->color.g, plot->color.b, plot->color.a);

//	glDisable (GL_LIGHTING);

	glBegin (GL_LINES);

		glLineWidth (plot->lineWidth);

		for (i=0; i <= grid->segments.z; i++)
		{
			for (j=0; j <= grid->segments.x; j++)
			{
				glVertex3f (x, 0.0f, z);
				glVertex3f (x, length.y, z);
				glVertex3f (0.0f, y, z);
				glVertex3f (length.x, y, z);
				x += grid->length.x;
				y += grid->length.y;
			}
			x = 0.0f;
			y = 0.0f;
			z += grid->length.z;
		}
	glEnd();

//	glEnable (GL_LIGHTING);

	glPopMatrix();
}


// draw 2D overlays including grids, text, messages, highlight objects, etc...
//------------------------------------------------------------------------------
void npDrawHUD (void* dataRef)
{

}


/////////////////////////////updates a single plot with new data if not frozen and draws it if not hidden
void SetIndexColor (NPubyteRGBA *color, int *colorIndex)
{	
	int index = *colorIndex;
	
	GLubyte colorPallet[kNPcolorIndexSize][4] = {	
											{255, 255, 255, 255},
											{0, 255, 0, 255},
											{255, 0, 0, 255},
											{0, 0, 255, 255},
											{255, 255, 0, 255},
											{152, 0, 255, 255},
											{255, 168, 0, 255},
											{0, 255, 255, 255},
											{255, 0, 255, 255},
											{0, 153, 0, 255},
											{173, 255, 202, 255},
											{255, 180, 255, 255},
											{0, 152, 255, 255},
											{185, 255, 0, 255},
											{152, 0, 0, 255},
											{50, 101, 101, 255},
											{127, 127, 255, 255},
											{185, 153, 102, 255},
											{197, 82, 0, 255},
											{127, 127, 127, 255}  };
											
	//make sure colorIndex is inbounds, if not, then put it inbounds																			
	if (index >= kNPcolorIndexSize)
		while (index >= kNPcolorIndexSize)
			index -= kNPcolorIndexSize;		//subtracts pallete sized increments
	
	if (index < 0)
		while (index < 0)
			index += kNPcolorIndexSize;		//adds pallete sized increments

	//now set the color
	color->r = colorPallet[index][0];
	color->g = colorPallet[index][1];
	color->b = colorPallet[index][2];
	color->a = colorPallet[index][3];		//perhaps better not to change alpha, debug

	*colorIndex = index;
}

